"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Api = void 0;
const websocket_1 = require("websocket");
const helper_1 = require("./helper");
const user_preferences_1 = require("./user_preferences");
const dayjs_1 = __importDefault(require("dayjs"));
const utc_1 = __importDefault(require("dayjs/plugin/utc"));
const timezone_1 = __importDefault(require("dayjs/plugin/timezone"));
const isBetween_1 = __importDefault(require("dayjs/plugin/isBetween"));
const relativeTime_1 = __importDefault(require("dayjs/plugin/relativeTime"));
const datetime_1 = require("./model/nested/datetime");
const ux_1 = require("./helpers/ux");
const log_1 = require("./helpers/log");
dayjs_1.default.extend(utc_1.default);
dayjs_1.default.extend(timezone_1.default);
dayjs_1.default.extend(isBetween_1.default);
dayjs_1.default.extend(relativeTime_1.default);
class Request {
    constructor(resolve, reject) {
        this.resolve = resolve;
        this.reject = reject;
    }
}
const RESPONSE_REGEX = new RegExp("([0-9]+):(.*)");
let instance = null;
class DefaultStore {
    constructor() {
        this.values = {};
    }
    setItem(key, value) {
        this.values[key] = value;
    }
    getItem(key) {
        return this.values[key];
    }
}
const internalStore = new DefaultStore();
const EVENT_TYPES = {
    ON_LOGIN: 'OnLogin',
    ON_TRIP_COUNTDOWN: 'OnTripCountDown',
    ON_NAVIGATE: 'navigate',
    ON_COMPANY_ACTIVATE: 'OnCompanyActivate',
    ON_UPDATE_IDENTITY: 'OnUpdateIdentity',
    ON_NEW_ALERT: 'OnNewAlert',
    ON_ALERTS_UPDATED: 'OnAlertsUpdate',
    ON_SHOW_ALERTS: "OnShowAlerts",
    ON_RESET_FILTER: "OnResetFilter",
    ON_REFRESH_DATA: 'OnRefreshData',
};
class Api {
    constructor() {
        this.requests = {};
        this.requestID = 0;
        this.connected = false;
        this.openSocketPromise = null;
        this.events = null;
        this.partialConnection = {};
        this.isAdmin = false;
        this.hostname = '';
        this.port = '';
        this.isSsl = false;
        this.localStorage = internalStore;
        this.sessionStorage = new DefaultStore();
        this.activeCompany = null;
        this.loginCompany = null;
        //
        //Process defaults.
        //Altered by NUXT and NODEJS 
        //on startup
        //
        this.distanceUnit = "mi";
        this.heightUnit = "in";
        this.weigthUnit = "lb";
        this.countryCode = 'United States';
        this.tz = "America/Los_Angeles";
        this.dtAppLongFormat = "DD-MMM-YYYY HH:mm";
        this.dtAppFormat = "DD-MM-YYYY HH:mm";
        this.dateAppFormat = "DD-MM-YYYY";
    }
    get EVENT_TYPES() {
        return EVENT_TYPES;
    }
    // End process defaults
    //
    get dayjs() {
        dayjs_1.default.tz.setDefault(this.tz);
        return dayjs_1.default;
    }
    get now() {
        dayjs_1.default.tz.setDefault(this.tz);
        return dayjs_1.default().tz(this.tz, true);
    }
    get nowDT() {
        let dt = new datetime_1.DateTime();
        dt.timezone = this.dayjs.tz.guess();
        dt.datetime = this.now.format(this.DTFormat);
        return dt;
    }
    get nowFmt() {
        return this.now.format(this.DTFormat);
    }
    get nowLocal() {
        return dayjs_1.default().tz(dayjs_1.default.tz.guess(), true);
    }
    get nowLocalDT() {
        let dt = new datetime_1.DateTime();
        dt.timezone = this.dayjs.tz.guess();
        dt.datetime = this.nowLocal.format(this.DTFormat);
        return dt;
    }
    ts2dt(timestamp) {
        let date = this.dayjs.tz(timestamp);
        let dt = new datetime_1.DateTime();
        dt.timezone = this.dayjs.tz.guess();
        dt.datetime = date.format(this.DTFormat);
        return dt;
    }
    asServerDT(dt) {
        dayjs_1.default.tz.setDefault(this.tz);
        return dayjs_1.default(dt).tz(this.tz, true);
    }
    fmtDT(dt, format) {
        dayjs_1.default.tz.setDefault(this.tz);
        return dayjs_1.default
            .tz(dt)
            .format(format ? format : this.dtAppFormat);
    }
    fmtDate(date, format) {
        dayjs_1.default.tz.setDefault(this.tz);
        return dayjs_1.default(date, this.DateFormat)
            .format(format ? format : this.dtAppFormat);
    }
    get DTFormat() {
        return this.DateFormat + "THH:mm";
    }
    get DateFormat() {
        return "YYYY-MM-DD";
    }
    notifyRefreshData() {
        this.notify(EVENT_TYPES.ON_REFRESH_DATA, true);
    }
    localDT(dt) {
        dayjs_1.default.tz.setDefault(this.tz);
        return dayjs_1.default
            .tz(dt)
            .tz(dayjs_1.default.tz.guess());
    }
    localDTFmt(dt, format) {
        dayjs_1.default.tz.setDefault(this.tz);
        return this.localDT(dt).format(format ? format : this.dtAppFormat);
    }
    localDateFmt(dt, format) {
        dayjs_1.default.tz.setDefault(this.tz);
        return this.localDT(dt).format(format ? format : this.dateAppFormat);
    }
    static instance() {
        if (instance == null) {
            instance = new Api();
        }
        return instance;
    }
    get organization() {
        return this.hostname.toLowerCase();
    }
    get domain() {
        let x = this.hostname.split(".");
        x.shift();
        return x.join(".");
    }
    uri() {
        if (this.isSsl) {
            return "wss://" + this.hostname + "/websocket";
        }
        else {
            return "ws://" + this.hostname + ":48080/websocket";
        }
    }
    getUriForHost(host) {
        let protocol = this.isSsl ? "https:" : "http:";
        let port = this.port || "";
        if (port != "") {
            port = ":" + port;
        }
        return protocol + "//" + host + port;
    }
    get isConnected() {
        return this.connected;
    }
    set isConnected(value) {
        //this.log("API.isConnected", value);
        this.connected = value;
        this.notify(this.EVENT_TYPES.ON_LOGIN, value);
    }
    get loginCompanyID() {
        return Number(this.sessionStorage.getItem('login_organization')) || -1;
    }
    set loginCompanyID(value) {
        let x = (typeof value == 'number') ? value : -1;
        this.sessionStorage.setItem("login_organization", x.toString());
    }
    get activeCompanyID() {
        return Number(this.sessionStorage.getItem('active_organization')) || -1;
    }
    set activeCompanyID(value) {
        let x = (typeof value == 'number') ? value : -1;
        this.sessionStorage.setItem("active_organization", x.toString());
    }
    set sessionIdentityID(value) {
        let x = (typeof value == 'number') ? value : -1;
        this.sessionStorage.setItem("identity_id", x.toString());
    }
    get sessionIdentityID() {
        return Number(this.sessionStorage.getItem("identity_id")) || -1;
    }
    get sessionUserName() {
        return this.sessionStorage.getItem("user") || "";
    }
    get sessionToken() {
        return this.sessionStorage.getItem('authtoken') || "";
    }
    set sessionToken(token) {
        this.sessionStorage.setItem('authtoken', token);
    }
    async setActiveCompanyID(value, navigate = true) {
        this.sessionStorage.setItem('active_organization', value);
        return new Promise((resolve, reject) => {
            if (this.isAdmin) {
                this.operation("company.activate", { company_id: value }).then(res => {
                    //this.log("API.setActiveCompanyID.company.activate", value);
                    this.activeCompany = res;
                    if (navigate) {
                        this.notify(this.EVENT_TYPES.ON_NAVIGATE, '/company-landing');
                    }
                    this.notify(this.EVENT_TYPES.ON_COMPANY_ACTIVATE, res);
                    resolve(res);
                }).catch(e => {
                    //this.log("API.setActiveCompanyID.isAdmin.error", e);
                    reject(null);
                });
            }
            else {
                this.operation("company.read", { company_id: Number(value) }).then(res => {
                    //this.log("API.setActiveCompanyID.company.read", value);
                    this.activeCompany = res;
                    if (navigate) {
                        this.notify(this.EVENT_TYPES.ON_NAVIGATE, '/company-landing');
                    }
                    this.notify(this.EVENT_TYPES.ON_COMPANY_ACTIVATE, res);
                    resolve(res);
                }).catch(e => {
                    //this.log("API.setActiveCompanyID.error", e);
                    reject(null);
                });
            }
        });
    }
    notify(name, value) {
        if (this.events) {
            this.events.$emit(name, value);
        }
    }
    close() {
        if (this.socket) {
            this.socket.close();
            //this.socket = null;
            this.openSocketPromise = null;
            this.resetSessionVariables();
        }
    }
    resetSessionVariables() {
        this.connected = false;
        this.requests.length = 0;
        this.activeCompany = null;
        this.loginCompany = null;
        this.sessionStorage.setItem('identity_id', "");
        this.sessionStorage.setItem('user', "");
        this.sessionToken = "";
        this.sessionStorage.setItem('login_organization', "");
        this.sessionStorage.setItem('active_organization', "");
    }
    prelogin() {
        return new Promise((resolve, reject) => {
            let socket = new websocket_1.w3cwebsocket(this.uri());
            socket.onopen = function () {
                this.send("prelogin,{}");
            };
            socket.onmessage = (e) => {
                //this.log("API.prelogin",e);
                let response = JSON.parse(e.data);
                if (response.data) {
                    let valid = (response.data.company_id >= 0);
                    if (valid) {
                        resolve({ validURI: valid });
                    }
                    else {
                        reject('/error/unknownuri');
                        this.notify(this.EVENT_TYPES.ON_NAVIGATE, '/error/unknownuri');
                    }
                }
                else {
                    reject('/error/401');
                    this.notify(this.EVENT_TYPES.ON_NAVIGATE, '/error/401');
                }
                socket.close();
            };
            socket.onerror = function (e) {
                reject(e);
                socket.close();
            };
        });
    }
    checkServerIsUp(retries = 30) {
        let that = this;
        function tick(resolve, reject, count) {
            that.prelogin().then(resolve).catch(() => {
                if (count === 0) {
                    reject();
                }
                else {
                    setTimeout(() => {
                        tick(resolve, reject, count - 1);
                    }, 1000);
                }
            });
        }
        return new Promise((resolve, reject) => {
            tick(resolve, reject, retries);
        });
    }
    activate(payload) {
        return new Promise((resolve, reject) => {
            let socket = new websocket_1.w3cwebsocket(this.uri());
            socket.onopen = function () {
                let s = JSON.stringify(payload);
                this.send("activate," + s);
            };
            socket.onmessage = (e) => {
                //this.log("API.activate.onmessage",e);
                if (e.data) {
                    let response = JSON.parse(e.data);
                    if (response.data) {
                        resolve(response.data);
                    }
                    else {
                        reject(response.error);
                    }
                }
                socket.close();
            };
            socket.onerror = function (e) {
                reject(e);
                socket.close();
            };
        });
    }
    _openSocket(operation, payload) {
        if (this.openSocketPromise) {
            //this.log("OPENSOCKET.OLDPROMISE",payload);
            return this.openSocketPromise;
        }
        //this.log("OPENSOCKET.NEWPROMISE",payload);
        this.openSocketPromise = new Promise((resolve, reject) => {
            this.socket = new websocket_1.w3cwebsocket(this.uri());
            this.socket.onopen = function () {
                this.send(operation + ", " + payload);
            };
            this.socket.onmessage = (e) => {
                //this.log("OPENSOCKET.ONMESSAGE",e);
                if (e.data) {
                    let response = JSON.parse(e.data);
                    this.socket.onmessage = (e) => this.handleMessage(e);
                    if (response.error) {
                        resolve({
                            response: false,
                            organization: this.organization,
                            status: response.status,
                            error: response.error
                        });
                    }
                    else if (response.data) {
                        let data = response.data;
                        //this.log("API._openSocket.onmessage", response);
                        resolve({
                            response: (response.status == "200"),
                            status: response.status,
                            type: data.type,
                            url: data.url,
                            user: data.user,
                            identity_id: data.identity_id,
                            organization: this.organization,
                            token: data.token,
                            timezone: data.timezone,
                        });
                    }
                }
                reject(e);
            };
            this.socket.onerror = function (e) {
                reject(e);
            };
        });
        return this.openSocketPromise;
    }
    _resumeLogin(user, token) {
        let payload = JSON.stringify({ user: user, token: token });
        return this._openSocket("resume", payload);
    }
    resumeLogin() {
        let token = this.sessionToken;
        let user = this.sessionUserName;
        return new Promise((resolve, reject) => {
            this._resumeLogin(user, token).then((res) => {
                if (res.response) {
                    //this.log("resumeLogin.timezone", res.timezone)
                    //this.timezone = res.timezone;
                    this.onLoginSuccessful(res.token, false).then(() => {
                        resolve(true);
                    }).catch(e => {
                        this.log("RESUMELOGIN.error.onLoginSuccessful", e);
                        this.notify(this.EVENT_TYPES.ON_NAVIGATE, '/');
                        resolve(false);
                    });
                }
                else {
                    this.notify(this.EVENT_TYPES.ON_NAVIGATE, '/');
                    resolve(false);
                }
            });
        });
    }
    get userAlerts() {
        let user = this.sessionStorage.getItem('user');
        let alerts = this.localStorage.getItem(user + "Alerts");
        if (alerts) {
            return JSON.parse(alerts);
        }
        else {
            return [];
        }
    }
    set userAlerts(alerts) {
        let user = this.sessionStorage.getItem('user');
        if (user != "") {
            this.localStorage.setItem(user + "Alerts", JSON.stringify(alerts));
            this.notify(EVENT_TYPES.ON_ALERTS_UPDATED, alerts);
        }
    }
    get userPreferences() {
        let user = this.sessionStorage.getItem('user');
        let pref = this.localStorage.getItem(user + "Preferences");
        let newPref = new user_preferences_1.UserPreferences();
        if (pref) {
            let x = JSON.parse(pref);
            return helper_1.Helper.instance().ObjectClone(newPref, x);
        }
        else {
            if (this.activeCompany) {
                newPref.navbar.show.unfulfilled = this.activeCompany.isCareProvider;
            }
        }
        return new user_preferences_1.UserPreferences();
    }
    set userPreferences(pref) {
        let user = this.sessionStorage.getItem('user');
        if (user != "") {
            this.localStorage.setItem(user + "Preferences", JSON.stringify(pref));
            this.notify(user_preferences_1.PREFERENCE_EVENTS.ON_UPDATED, pref);
        }
    }
    login(username, password, token) {
        this.resetSessionVariables();
        let req = { user: username.toLowerCase() };
        if (token) {
            req.passwords = [password, token];
        }
        else {
            req.password = password;
        }
        let payload = JSON.stringify(req);
        return this._openSocket("login", payload).then(result => {
            this.sessionStorage.setItem("user", username);
            if (result.response) {
                //this.log("API.login", result);
                this.sessionIdentityID = result.identity_id;
                return this.onLoginSuccessful(result.token, true);
            }
            else {
                result.username = username;
                return this.onLoginFailed(result);
            }
        });
    }
    logout() {
        return new Promise((resolve, reject) => {
            this._openSocket("logout", "").then((res) => {
                this.close();
                resolve(res);
            }).catch((e) => {
                this.log("API.logout.error", e);
                reject(e);
            });
        });
    }
    navigateHome() {
        this.notify(this.EVENT_TYPES.ON_NAVIGATE, '/');
    }
    navigateAdminLanding() {
        if (this.isAdmin) {
            this.notify(this.EVENT_TYPES.ON_NAVIGATE, '/admin-landing');
        }
    }
    continueLogin(parameters) {
        //this.log("API.continueLogin",parameters);
        return new Promise((resolve, reject) => {
            let newID = this.requestID++;
            this.requests[newID] = new Request(resolve, reject);
            let x = newID + ": continue," + JSON.stringify(parameters);
            let state = this.socket.readyState;
            if (state == 1) {
                this.socket.send(x);
            }
            else {
                //this.log("API.continueLogin.INVALIDSOCKETSTATE",state);
                this.requests[newID].reject({ message: "Invalid socket state.", socketState: state });
                this.deleteRequest(newID);
            }
        });
    }
    isTrue(value) {
        return value ? (value == true || value == "true") : false;
    }
    onLoginSuccessful(token, navigate) {
        this.sessionToken = token;
        let search = { view: 'company.readbyuri', params: [this.organization] };
        return new Promise((resolve, reject) => {
            this.operation('query.search', search).then((results) => {
                let res = results[0];
                //this.log("API.onLoginSuccessful", res);
                this.loginCompany = res;
                this.sessionStorage.setItem('login_organization', res.company_id);
                this.isAdmin = this.isTrue(res.company_type_properties.admin);
                //Call login event
                this.isConnected = true;
                let activeID = this.sessionStorage.getItem('active_organization');
                let isValidActiveID = (activeID && activeID != "null" && activeID != "");
                this.setActiveCompanyID(isValidActiveID ? activeID : res.company_id, false).then(r => {
                    //this.log("API.onLoginSuccessful.setActiveCompanyID", r);
                    if (navigate) {
                        if (this.isAdmin) {
                            this.notify(this.EVENT_TYPES.ON_NAVIGATE, '/admin-landing');
                        }
                        else {
                            this.notify(this.EVENT_TYPES.ON_NAVIGATE, '/company-landing');
                        }
                    }
                    resolve({ user: this.sessionUserName, identity_id: this.sessionIdentityID, company: this.activeCompany });
                }).catch(error => {
                    reject(error);
                });
            }).catch(error => {
                reject(error);
            });
        });
    }
    onLoginFailed(result) {
        return new Promise((resolve, reject) => {
            this.sessionToken = result.token;
            if (result.status == "202") {
                this.partialConnection = result;
                if (result.url) {
                    this.notify(this.EVENT_TYPES.ON_NAVIGATE, '/mfa/register');
                }
                else {
                    this.notify(this.EVENT_TYPES.ON_NAVIGATE, '/mfa/mflogin');
                }
                resolve({ message: "ok" });
            }
            else if (result.error) {
                this.close();
                reject({ message: result.error.message ? result.error.message : "" });
            }
        });
    }
    log(label, data) {
        console.log({ label: label, log: data });
    }
    handleMessage(e) {
        //this.log("ONHANDLEMESSAGE", { msg: e, socket: this.socket });
        if (e.data && (typeof e.data === 'string')) {
            var id;
            if (e.data.startsWith("ERROR")) {
                let parts = e.data.split(":", 3);
                id = parts[1];
                this.requests[id].reject(parts[2]);
            }
            else {
                let m = e.data.match(/^([0-9]+)\:([\s\S]*)/m);
                if (m) {
                    id = m[1];
                    try {
                        let d = JSON.parse(m[2]);
                        //this.log("ONHANDLEMESSAGE.OK", { msg: d, socket: this.socket });
                        if (d.status == "202") {
                            this.requests[id].reject({ message: "Invalid Token" });
                        }
                        else if (d.status == "401") {
                            //this.log("ONHANDLEMESSAGE.401", { id: id, msg: d, r: this.requests });
                            d.error.failedLogin = true;
                            this.requests[id].reject(d.error);
                        }
                        else if (d.error) {
                            this.requests[id].reject(d.error);
                        }
                        else if (d.data || d.status == "200") {
                            this.requests[id].resolve(d.data);
                        }
                    }
                    catch (e) {
                        this.log("ONHANDLEMESSAGE.PARSEEXCEPTION", m[2]);
                        this.requests[id].reject(e);
                    }
                }
                else {
                    this.handleMaybeNotificationMessage(e.data);
                }
            }
            this.deleteRequest(id);
        }
    }
    handleMaybeNotificationMessage(data) {
        try {
            let event = JSON.parse(data);
            if (event) {
                this.log("API.NEW_ALERT.preprocessed", JSON.parse(data).data);
                event.data.dt = this.nowLocalDT;
                event.data.event = event.data.event.toUpperCase();
                event.data.event_entity = event.data.entity_type.toUpperCase();
                event.data.offer_tier = "";
                event.data.expanded = false;
                event.data.ux = ux_1.UXHelper.getEventUX(event.data);
                try {
                    event.data.comment = event.data.message;
                    log_1.TripLogHelper.processLogComments(event.data);
                    //if (event.data.message && !Array.isArray(event.data.message)) {
                    //    event.data.message = event.data.message.replaceAll('"', "").split("\n");
                    //}
                }
                catch (e) {
                }
                let alerts = this.userAlerts;
                alerts.push(event.data);
                this.userAlerts = alerts;
                this.log("API.NEW_ALERT.processed", event.data);
                this.notify(this.EVENT_TYPES.ON_NEW_ALERT, event.data);
            }
        }
        catch (error) {
            this.log("HANDLEMESSAGE.unhandled", { data: data, error: error.message });
        }
    }
    deleteRequest(id) {
        delete this.requests[id];
    }
    operation(operation, parameters) {
        return new Promise((resolve, reject) => {
            this.requestID++;
            this.requests[this.requestID] = new Request(resolve, reject);
            let x = this.requestID + ":" + operation + "," + JSON.stringify(parameters);
            this.socket.send(x);
        });
    }
    query(name, parameters) {
        let p = [];
        let view = { view: name, params: parameters };
        return this.operation("query.search", view);
    }
    queryPaginated(name, pagination) {
        let view = { view: name, options: pagination };
        return this.operation("query.paginated_search", view);
    }
    search(name, options, parameters) {
        let json = options;
        if (parameters != undefined) {
            json.params = parameters;
        }
        return this.operation("query.paginated_search", json);
    }
    queryWithMap(name, filters) {
        let view = { view: name, params: filters };
        return this.operation("query.search", view);
    }
    searchWithMap(name, options, parameters) {
        let json = options;
        if (parameters != undefined) {
            json.params = parameters;
        }
        return this.operation("query.paginated_search", json);
    }
    sendEmailTemplate(to, from, template, data) {
        //
        let payload = {
            to: to,
            from: from,
            templateid: template,
            templatedata: data
        };
        return this.operation("notification.email", payload);
    }
    sendACCNewAccountEmail(to, username, uri, expiration) {
        //
        //this.log("API.sendACCNewAccountEmail",{username:username,confirmation_uri: uri} );
        return this.sendEmailTemplate(to, "omasri@mamori.io", "d-69579f50d10a4d37bc040e8665e6540e", { username: username, confirmation_uri: uri, expiration_date: expiration });
    }
    read(operation, id) {
        return new Promise((resolve, reject) => {
            this.requestID++;
            this.requests[this.requestID] = new Request(resolve, reject);
            this.socket.send(this.requestID + ":" + operation + "," + id);
        });
    }
    create(operation, entity) {
        return new Promise((resolve, reject) => {
            this.requestID++;
            this.requests[this.requestID] = new Request(resolve, reject);
            this.socket.send(this.requestID + ":" + operation + "," + JSON.stringify(entity));
        });
    }
    update(operation, entity) {
        return new Promise((resolve, reject) => {
            this.requestID++;
            this.requests[this.requestID] = new Request(resolve, reject);
            this.socket.send(this.requestID + ":" + operation + "," + JSON.stringify(entity));
        });
    }
    upsert(operation, entity, reference) {
        return new Promise((resolve, reject) => {
            this.requestID++;
            this.requests[this.requestID] = new Request(resolve, reject);
            let json = JSON.stringify(entity);
            if (reference != undefined) {
                let data = { entity: entity, reference: reference };
                json = JSON.stringify(data);
            }
            this.socket.send(this.requestID + ":" + operation + "," + json);
        });
    }
    delete(operation, id) {
        return new Promise((resolve, reject) => {
            this.requestID++;
            this.requests[this.requestID] = new Request(resolve, reject);
            this.socket.send(this.requestID + ":" + operation + "," + id);
        });
    }
    async callTypes() {
        return await this.operation("calltype.readall");
    }
}
exports.Api = Api;
//# sourceMappingURL=api.js.map