"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.instance = exports.IdentityHelper = void 0;
const dialogs_1 = require("../dialogs");
const api_1 = require("../api");
const helper_1 = require("../helper");
const company_1 = require("../model/company");
const internal_user_1 = require("../model/internal_user");
class RolesStates {
    constructor() {
        this.acc_admin = { name: "acc_admin", has: true, };
        this.company_admin = { name: "company_admin", has: true };
        this.company_config_manager = { name: "company_config_manager", has: true };
        this.dispatch = { name: "dispatch", has: true };
        this.manage_patient = { name: "manage_patient", has: true };
        this.manage_booking = { name: "manage_booking", has: true };
    }
    get canManageConfig() {
        return this.acc_admin.has || this.company_config_manager.has;
    }
    get canManageAccounts() {
        return this.acc_admin.has || this.company_admin.has;
    }
    get canManagePatients() {
        return this.acc_admin.has || this.company_config_manager.has || this.manage_patient.has;
    }
    get canManageBookings() {
        return this.acc_admin.has || this.company_config_manager.has || this.manage_booking.has;
    }
}
class MenuStates {
    constructor(roles, company) {
        this.roles = roles;
        this.company = company;
    }
    get users() {
        return this.roles.canManageConfig;
    }
    get markets() {
        return this.roles.canManageConfig;
    }
    get marketRules() {
        return this.roles.canManageConfig;
    }
    get bookings() {
        return this.roles.canManageBookings;
    }
}
class IdentityHelper {
    constructor() {
        this.api = api_1.Api.instance();
        this.helper = helper_1.Helper.instance();
        this.user = null;
        this.dbRoles = null;
        this.roles = new RolesStates();
        this.menus = new MenuStates(this.roles, this.api.activeCompany);
    }
    static instance() {
        return exports.instance;
    }
    roleCheck(role) {
        return this.dbRoles ?
            this.dbRoles.filter((p) => { return p.role_name == role; }).length > 0
            : false;
    }
    refreshStates() {
        Object.keys(this.roles).forEach(key => {
            let role = this.roles[key];
            role.has = this.roleCheck(role.name);
        });
    }
    get roleSummary() {
        let res = [];
        if (this.roles.canManageAccounts && this.roles.canManageConfig) {
            res.push(this.roles.acc_admin.has ? "ACC Super Admin" : "Super Admin");
        }
        else if (this.roles.canManageConfig) {
            res.push(this.roles.acc_admin.has ? "ACC Config Admin" : "Config Admin");
        }
        else if (this.roles.canManageAccounts) {
            res.push(this.roles.acc_admin.has ? "ACC Account Admin" : "Account Admin");
        }
        if (this.roles.canManagePatients && this.roles.canManageBookings) {
            res.push("Case Manager");
        }
        else if (this.roles.canManagePatients) {
            res.push("Patient Manager");
        }
        else if (this.roles.canManagePatients) {
            res.push("Booking Manager");
        }
        if (this.roles.dispatch) {
            res.push("Dispatcher");
        }
        return res;
    }
    get grantedRoles() {
        if (!this.dbRoles) {
            let p = this.api.sessionStorage.getItem('grantedRoles');
            if (p) {
                this.dbRoles = JSON.parse(p);
            }
        }
        return this.dbRoles;
    }
    set grantedRoles(roles) {
        this.dbRoles = roles;
        this.api.sessionStorage.setItem('grantedRoles', JSON.stringify(roles));
    }
    async addRolesIfNotExist(identity_id, roles, targets) {
        return new Promise((resolve, reject) => {
            this.api.operation("identity.grantedroles", {
                identity_id: identity_id,
                inherited: false
            }).then((granted) => {
                let grants = [];
                for (let role of roles) {
                    let match = granted.filter((p) => { return p.role_name == role; }).length > 0;
                    if (!match) {
                        grants.push(this.grantRoleByName(identity_id, role, targets));
                    }
                }
                Promise.all(grants).then(res => {
                    this.api.operation("identity.grantedroles", {
                        identity_id: identity_id, inherited: false
                    }).then(result => {
                        resolve(result);
                    });
                });
            }).catch(e => {
                reject(e);
            });
        });
    }
    async refreshIdentityDetails(company) {
        return new Promise((resolve, reject) => {
            this.dbRoles = null;
            this.user = null;
            let identityCompany = company ? company : this.api.loginCompany;
            this.menus.company = identityCompany;
            if (this.api.sessionIdentityID > -1) {
                this.api.operation("identity.grantedroles", {
                    identity_id: this.api.sessionIdentityID, inherited: false
                }).then(roles => {
                    this.grantedRoles = roles;
                    this.refreshStates();
                    this.api.query("internal_user.by_identity", [])
                        .then(result => {
                        if (result.length == 1) {
                            this.user = this.helper.ObjectClone(new internal_user_1.InternalUser(), result[0]);
                            resolve(this.user);
                        }
                        else {
                            reject({ message: "internal user not found for identity", data: this.api.sessionIdentityID });
                        }
                    }).catch(error => {
                        reject(error);
                    });
                }).catch(error => {
                    reject(error);
                });
            }
            else {
                reject({ message: "invalid session identity id", data: this.api.sessionIdentityID });
            }
        });
        //"identity.grantedroles"
        //identity.allowed_values
    }
    async createAPILogin(user, company) {
        let c = company ? company : this.api.activeCompany;
        return this.createCompanyLogin(user, c, 'api_uri');
    }
    async createAppLogin(user, company) {
        let c = company ? company : this.api.activeCompany;
        return this.createCompanyLogin(user, c, 'user_uri');
    }
    getCompanyProvider(company, uriType) {
        return new Promise((resolve, reject) => {
            let that = this;
            function doIt(co) {
                let uri = co.properties[uriType];
                that.api.operation("company_provider_link.readall", { company_id: co.id, uri: uri }).then(providers => {
                    if (providers.length > 0) {
                        resolve(providers[0]);
                    }
                    reject({ message: "No " + uriType + " provider found" });
                }).catch(error => reject(error));
            }
            if (company) {
                doIt(company);
            }
            else {
                //this.api.operation("company.read", { company_id: this.api.activeCompanyID })
                company_1.companyservice.read(new company_1.Company(), this.api.activeCompanyID).then(company => {
                    doIt(company);
                }).catch(error => reject(error));
            }
        });
    }
    createCompanyLogin(user, company, uriType) {
        return new Promise((resolve, reject) => {
            this.getCompanyProvider(company, uriType).then(provider => {
                if (provider) {
                    user.providerId = provider.provider_id;
                    internal_user_1.internal_userservice.create(user).then(newuser => {
                        resolve(newuser);
                    }).catch(error => reject(error));
                }
            }).catch(error => reject(error));
        });
    }
    companyLoginFind(name, company, uriType) {
        return new Promise((resolve, reject) => {
            this.getCompanyProvider(company, uriType).then(provider => {
                if (provider) {
                    let pid = provider.provider_id;
                    internal_user_1.internal_userservice.readAll(new internal_user_1.InternalUser(), { internal_user_provider_id: pid, internal_user_name: name }).then(result => {
                        if (result.length > 0) {
                            resolve(result[0]);
                        }
                        resolve(null);
                    }).catch(error => reject(error));
                }
            }).catch(error => reject(error));
        });
    }
    companyLoginReadAll(company, uriType) {
        return new Promise((resolve, reject) => {
            this.getCompanyProvider(company, uriType).then(provider => {
                if (provider) {
                    let pid = provider.provider_id;
                    return this.loginsReadAll(pid);
                }
            }).catch(error => reject(error));
        });
    }
    loginsByIdentity(provider_id, identity_id) {
        return new Promise((resolve, reject) => {
            internal_user_1.internal_userservice.readAll(new internal_user_1.InternalUser(), { internal_user_provider_id: provider_id, identity_id: identity_id }).then(result => {
                resolve(result);
            }).catch(error => reject(error));
        });
    }
    loginsReadAll(provider_id) {
        return new Promise((resolve, reject) => {
            internal_user_1.internal_userservice.readAll(new internal_user_1.InternalUser(), { internal_user_provider_id: provider_id }).then(result => {
                resolve(result);
            }).catch(error => reject(error));
        });
    }
    grantRoleByName(identityID, roleName, facilityTargets) {
        return new Promise((resolve, reject) => {
            this.api.operation("role.ui.readall", {}).then((data) => {
                let roles = data.filter((item) => {
                    return item.role_name == roleName;
                });
                if (roles.length > 0) {
                    this.grantRoleById(identityID, roles[0].role_id, facilityTargets).then(result => {
                        resolve(result);
                    }).catch(e => reject(e));
                }
                else {
                    reject("Role not found.");
                }
            }).catch(e => {
                dialogs_1.instance.alert("Error", e.message);
                reject(e);
            });
        });
    }
    grantRoleById(identityID, roleID, facilityTargets) {
        let payload = {
            role_id: roleID,
            identity_id: identityID
        };
        if (facilityTargets) {
            payload.targets = facilityTargets;
        }
        return this.api.operation("identity.grant.role", payload).catch(e => {
            dialogs_1.instance.alert("Error", e.message);
        });
    }
    revokeRoleById(identityID, roleID) {
        let payload = {
            role_id: roleID,
            identity_id: identityID
        };
        return this.api.operation("identity.revoke.role", payload).catch(e => {
            dialogs_1.instance.alert("Error", e.message);
        });
    }
    identityGrantedRoles(identity_id) {
        return this.api
            .operation("identity.grantedroles", {
            identity_id: identity_id,
            inherited: false
        }).catch(e => {
            dialogs_1.instance.alert("Error", e.message);
        });
    }
    allRoles() {
        return this.api.operation("role.ui.readall");
    }
    sendUserActivationEmailNotification(host, user, message) {
        return new Promise((resolve, reject) => {
            if (user.properties.activation_token) {
                let uri = this.api.getUriForHost(host) + user.getActivationURI();
                let udate = this.api.fmtDT(user.tokenExpiration);
                this.api.sendACCNewAccountEmail(user.properties.contact.email, user.name, uri, udate).then(result => {
                    let msg = message ? message : '';
                    dialogs_1.instance.alert("Alert", msg +
                        "Email notification sent to <br><b>user:</b>" +
                        user.name +
                        "<br><b>email:</b>" +
                        user.properties.contact.email);
                    resolve(true);
                }).catch((e) => {
                    dialogs_1.instance.alert("Error", e.message);
                    reject({ error: e.message });
                });
            }
            else {
                if (user.generatePW == "true") {
                    reject({ error: "missing user activation token" });
                }
                else {
                    resolve(true);
                }
            }
        });
    }
    users(company_provider_uri) {
        return new Promise((resolve, reject) => {
            this.api.query("internal_user.by_uri", [company_provider_uri])
                .then(result => {
                let users = result.map((user) => {
                    let o = this.helper.ObjectClone(new internal_user_1.InternalUser(), user);
                    let activation_token = o.properties.activation_token;
                    if (activation_token &&
                        typeof activation_token === "string") {
                        o.properties.activation_token = JSON.parse(activation_token);
                        //this.$Api.log("USERS.LOAD.INVALID_ACTIVATIONTOKEN", o);
                    }
                    else if (activation_token) {
                        let x = new Date(activation_token.expiration);
                        let comp = new Date() > x;
                        if (comp) {
                            o.token = "Expired";
                        }
                        else {
                            o.token =
                                "Valid until " +
                                    this.api.localDTFmt(activation_token.expiration);
                        }
                    }
                    return o;
                });
                resolve(users);
            }).catch(error => {
                reject(error);
            });
        });
    }
}
exports.IdentityHelper = IdentityHelper;
exports.instance = new IdentityHelper();
//# sourceMappingURL=identity.js.map